@@ -22,17 +22,6 @@
 #include "storage/s_lock.h"
 
 
-/*
- * CitusInitiatedBackend keeps some information about the backends that are
- * initiated by Citus.
- */
-typedef struct CitusInitiatedBackend
-{
-	int initiatorNodeIdentifier;
-	bool transactionOriginator;
-} CitusInitiatedBackend;
-
-
 /*
  * Each backend's active distributed transaction information is tracked via
  * BackendData in shared memory.
@@ -47,30 +36,49 @@ typedef struct CitusInitiatedBackend
 typedef struct BackendData
 {
 	Oid databaseId;
-	Oid userId;
 	slock_t mutex;
 	bool cancelledDueToDeadlock;
-	CitusInitiatedBackend citusBackend;
+	uint64 globalPID;
+	bool distributedCommandOriginator;
 	DistributedTransactionId transactionId;
+	bool activeBackend; /* set to false when backend exists */
 } BackendData;
 
 
+extern void BackendManagementShmemInit(void);
+extern size_t BackendManagementShmemSize(void);
 extern void InitializeBackendManagement(void);
 extern int TotalProcCount(void);
-extern void InitializeBackendData(void);
+extern void InitializeBackendData(const char *applicationName);
 extern void LockBackendSharedMemory(LWLockMode lockMode);
 extern void UnlockBackendSharedMemory(void);
 extern void UnSetDistributedTransactionId(void);
+extern void UnSetGlobalPID(void);
+extern void SetActiveMyBackend(bool value);
 extern void AssignDistributedTransactionId(void);
-extern void MarkCitusInitiatedCoordinatorBackend(void);
+extern void AssignGlobalPID(const char *applicationName);
+extern uint64 GetGlobalPID(void);
+extern void SetBackendDataDatabaseId(void);
+extern void SetBackendDataGlobalPID(uint64 gpid);
+extern uint64 ExtractGlobalPID(const char *applicationName);
+extern int ExtractNodeIdFromGlobalPID(uint64 globalPID, bool missingOk);
+extern int ExtractProcessIdFromGlobalPID(uint64 globalPID);
 extern void GetBackendDataForProc(PGPROC *proc, BackendData *result);
 extern void CancelTransactionDueToDeadlock(PGPROC *proc);
 extern bool MyBackendGotCancelledDueToDeadlock(bool clearState);
 extern bool MyBackendIsInDisributedTransaction(void);
 extern List * ActiveDistributedTransactionNumbers(void);
 extern LocalTransactionId GetMyProcLocalTransactionId(void);
-extern int GetAllActiveClientBackendCount(void);
-extern void IncrementClientBackendCounter(void);
-extern void DecrementClientBackendCounter(void);
+extern int GetExternalClientBackendCount(void);
+extern uint32 IncrementExternalClientBackendCounter(void);
+extern void DecrementExternalClientBackendCounter(void);
+extern void DetermineCitusBackendType(const char *applicationName);
+extern bool IsCitusInternalBackend(void);
+extern bool IsRebalancerInternalBackend(void);
+extern bool IsCitusRunCommandBackend(void);
+extern bool IsExternalClientBackend(void);
+
+#define INVALID_CITUS_INTERNAL_BACKEND_GPID 0
+#define GLOBAL_PID_NODE_ID_FOR_NODES_NOT_IN_METADATA 99999999
 
 #endif /* BACKEND_DATA_H */
