@@ -21,6 +21,7 @@
 #include "nodes/pg_list.h"
 #include "distributed/metadata_utility.h"
 
+#include "columnar/columnar.h"
 
 /*
  * In our distributed database, we need a mechanism to make remote procedure
@@ -46,14 +47,11 @@
 #define CANDIDATE_NODE_FIELDS 2
 #define WORKER_NODE_FIELDS 2
 
-/* transfer mode for master_copy_shard_placement */
+/* transfer mode for citus_copy_shard_placement */
 #define TRANSFER_MODE_AUTOMATIC 'a'
 #define TRANSFER_MODE_FORCE_LOGICAL 'l'
 #define TRANSFER_MODE_BLOCK_WRITES 'b'
 
-/* Name of columnar foreign data wrapper */
-#define CSTORE_FDW_NAME "cstore_fdw"
-
 #define SHARDID_SEQUENCE_NAME "pg_dist_shardid_seq"
 #define PLACEMENTID_SEQUENCE_NAME "pg_dist_placement_placementid_seq"
 
@@ -62,8 +60,6 @@
 	"SELECT worker_apply_shard_ddl_command (" UINT64_FORMAT ", %s, %s)"
 #define WORKER_APPLY_SHARD_DDL_COMMAND_WITHOUT_SCHEMA \
 	"SELECT worker_apply_shard_ddl_command (" UINT64_FORMAT ", %s)"
-#define WORKER_APPEND_TABLE_TO_SHARD \
-	"SELECT worker_append_table_to_shard (%s, %s, %s, %u)"
 #define WORKER_APPLY_INTER_SHARD_DDL_COMMAND \
 	"SELECT worker_apply_inter_shard_ddl_command (" UINT64_FORMAT ", %s, " UINT64_FORMAT \
 	", %s, %s)"
@@ -74,15 +70,6 @@
 #define DROP_FOREIGN_TABLE_COMMAND "DROP FOREIGN TABLE IF EXISTS %s CASCADE"
 #define CREATE_SCHEMA_COMMAND "CREATE SCHEMA IF NOT EXISTS %s AUTHORIZATION %s"
 
-/* Enumeration that defines the shard placement policy to use while staging */
-typedef enum
-{
-	SHARD_PLACEMENT_INVALID_FIRST = 0,
-	SHARD_PLACEMENT_LOCAL_NODE_FIRST = 1,
-	SHARD_PLACEMENT_ROUND_ROBIN = 2,
-	SHARD_PLACEMENT_RANDOM = 3
-} ShardPlacementPolicyType;
-
 /*
  * TableDDLCommandType encodes the implementation used by TableDDLCommand. See comments in
  * TableDDLCpmmand for details.
@@ -130,6 +117,18 @@ typedef enum IncludeSequenceDefaults
 } IncludeSequenceDefaults;
 
 
+/*
+ * IncludeIdentities decides on how we include identity information
+ * when creating the definition of a table.
+ */
+typedef enum IncludeIdentities
+{
+	NO_IDENTITY = 0, /* don't include identities */
+	INCLUDE_IDENTITY_AS_SEQUENCE_DEFAULTS = 1, /* include identities as sequences */
+	INCLUDE_IDENTITY = 2 /* include identities as-is*/
+} IncludeIdentities;
+
+
 struct TableDDLCommand;
 typedef struct TableDDLCommand TableDDLCommand;
 typedef char *(*TableDDLFunction)(void *context);
@@ -184,6 +183,17 @@ struct TableDDLCommand
 	};
 };
 
+/*
+ * ColumnarTableDDLContext holds the instance variable for the TableDDLCommandFunction
+ * instance described below.
+ */
+typedef struct ColumnarTableDDLContext
+{
+	char *schemaName;
+	char *relationName;
+	ColumnarOptions options;
+} ColumnarTableDDLContext;
+
 /* make functions for TableDDLCommand */
 extern TableDDLCommand * makeTableDDLCommandString(char *commandStr);
 extern TableDDLCommand * makeTableDDLCommandFunction(TableDDLFunction function,
@@ -193,14 +203,16 @@ extern TableDDLCommand * makeTableDDLCommandFunction(TableDDLFunction function,
 
 extern char * GetShardedTableDDLCommand(TableDDLCommand *command, uint64 shardId,
 										char *schemaName);
+extern char * GetShardedTableDDLCommandColumnar(uint64 shardId, void *context);
 extern char * GetTableDDLCommand(TableDDLCommand *command);
+extern TableDDLCommand * ColumnarGetCustomTableOptionsDDL(char *schemaName,
+														  char *relationName,
+														  ColumnarOptions options);
 
 
 /* Config variables managed via guc.c */
 extern int ShardCount;
 extern int ShardReplicationFactor;
-extern int ShardMaxSize;
-extern int ShardPlacementPolicy;
 extern int NextShardId;
 extern int NextPlacementId;
 
@@ -208,17 +220,21 @@ extern int NextPlacementId;
 extern bool IsCoordinator(void);
 
 /* Function declarations local to the distributed module */
-extern bool CStoreTable(Oid relationId);
 extern uint64 GetNextShardId(void);
 extern uint64 GetNextPlacementId(void);
 extern Oid ResolveRelationId(text *relationName, bool missingOk);
 extern List * GetFullTableCreationCommands(Oid relationId,
-										   IncludeSequenceDefaults includeSequenceDefaults);
+										   IncludeSequenceDefaults includeSequenceDefaults,
+										   IncludeIdentities includeIdentityDefaults,
+										   bool creatingShellTableOnRemoteNode);
 extern List * GetPostLoadTableCreationCommands(Oid relationId, bool includeIndexes,
 											   bool includeReplicaIdentity);
 extern List * GetPreLoadTableCreationCommands(Oid relationId, IncludeSequenceDefaults
 											  includeSequenceDefaults,
+											  IncludeIdentities
+											  includeIdentityDefaults,
 											  char *accessMethod);
+extern List * GetTableRowLevelSecurityCommands(Oid relationId);
 extern List * GetTableIndexAndConstraintCommands(Oid relationId, int indexFlags);
 extern List * GetTableIndexAndConstraintCommandsExcludingReplicaIdentity(Oid relationId,
 																		 int indexFlags);
@@ -268,9 +284,7 @@ extern Datum master_stage_shard_placement_row(PG_FUNCTION_ARGS);
 
 /* Function declarations to help with data staging and deletion */
 extern Datum master_create_empty_shard(PG_FUNCTION_ARGS);
-extern Datum master_append_table_to_shard(PG_FUNCTION_ARGS);
 extern Datum master_update_shard_statistics(PG_FUNCTION_ARGS);
-extern Datum master_apply_delete_command(PG_FUNCTION_ARGS);
 extern Datum master_drop_sequences(PG_FUNCTION_ARGS);
 extern Datum master_modify_multiple_shards(PG_FUNCTION_ARGS);
 extern Datum lock_relation_if_exists(PG_FUNCTION_ARGS);
@@ -282,13 +296,10 @@ extern int MasterDropAllShards(Oid relationId, char *schemaName, char *relationN
 extern Datum master_create_worker_shards(PG_FUNCTION_ARGS);
 extern Datum isolate_tenant_to_new_shard(PG_FUNCTION_ARGS);
 
-/* function declarations for shard repair functionality */
-extern Datum master_copy_shard_placement(PG_FUNCTION_ARGS);
+/* function declarations for shard split functionality */
+extern Datum citus_split_shard_by_split_points(PG_FUNCTION_ARGS);
 
 /* function declarations for shard copy functinality */
-extern List * CopyShardCommandList(ShardInterval *shardInterval, const
-								   char *sourceNodeName,
-								   int32 sourceNodePort, bool includeData);
 extern List * CopyShardForeignConstraintCommandList(ShardInterval *shardInterval);
 extern void CopyShardForeignConstraintCommandListGrouped(ShardInterval *shardInterval,
 														 List **
@@ -305,7 +316,6 @@ extern void ErrorIfTargetNodeIsNotSafeToMove(const char *targetNodeName, int
 extern char LookupShardTransferMode(Oid shardReplicationModeOid);
 extern void BlockWritesToShardList(List *shardList);
 extern List * WorkerApplyShardDDLCommandList(List *ddlCommandList, int64 shardId);
-extern List * GetForeignConstraintCommandsToReferenceTable(ShardInterval *shardInterval);
 
 
 #endif   /* COORDINATOR_PROTOCOL_H */
