@@ -16,8 +16,8 @@
 #include "storage/lockdefs.h"
 #include "distributed/multi_physical_planner.h"
 
-extern void AcquireExecutorShardLocks(Task *task, RowModifyLevel modLevel);
-extern void AcquireExecutorMultiShardLocks(List *taskList);
+extern void AcquireExecutorShardLocksForRelationRowLockList(List *relationRowLockList);
+extern bool RequiresConsistentSnapshot(Task *task);
 extern void AcquireMetadataLocks(List *taskList);
 extern void LockPartitionsInRelationList(List *relationIdList, LOCKMODE lockmode);
 extern void LockPartitionRelations(Oid relationId, LOCKMODE lockMode);
