@@ -19,7 +19,8 @@
 extern bool EnableLocalExecution;
 extern bool LogLocalCommands;
 
-extern int LocalExecutorLevel;
+/* global variable that tracks whether the local execution is on a shard */
+extern uint64 LocalExecutorShardId;
 
 typedef enum LocalExecutionStatus
 {
@@ -43,6 +44,7 @@ extern void ExecuteUtilityCommand(const char *utilityCommand);
 extern bool ShouldExecuteTasksLocally(List *taskList);
 extern bool AnyTaskAccessesLocalNode(List *taskList);
 extern bool TaskAccessesLocalNode(Task *task);
+extern void EnsureCompatibleLocalExecutionState(List *taskList);
 extern void ErrorIfTransactionAccessedPlacementsLocally(void);
 extern void DisableLocalExecution(void);
 extern void SetLocalExecutionStatus(LocalExecutionStatus newStatus);
