@@ -63,6 +63,7 @@ typedef struct
 	char partitionMethod;
 	uint32 colocationId;
 	char replicationModel;
+	bool autoConverted; /* table auto-added to metadata, valid for citus local tables */
 
 	/* pg_dist_shard metadata (variable-length ShardInterval array) for this table */
 	int shardIntervalArrayLength;
@@ -114,6 +115,7 @@ typedef struct DistObjectCacheEntry
 
 	int distributionArgIndex;
 	int colocationId;
+	bool forceDelegation;
 } DistObjectCacheEntry;
 
 typedef enum
@@ -141,16 +143,24 @@ extern List * AllCitusTableIds(void);
 extern bool IsCitusTableType(Oid relationId, CitusTableType tableType);
 extern bool IsCitusTableTypeCacheEntry(CitusTableCacheEntry *tableEtnry,
 									   CitusTableType tableType);
+extern char * GetTableTypeName(Oid tableId);
 
+extern void SetCreateCitusTransactionLevel(int val);
+extern int GetCitusCreationLevel(void);
 extern bool IsCitusTable(Oid relationId);
+extern bool IsCitusTableViaCatalog(Oid relationId);
 extern char PgDistPartitionViaCatalog(Oid relationId);
 extern List * LookupDistShardTuples(Oid relationId);
 extern char PartitionMethodViaCatalog(Oid relationId);
+extern Var * PartitionColumnViaCatalog(Oid relationId);
+extern uint32 ColocationIdViaCatalog(Oid relationId);
 extern bool IsCitusLocalTableByDistParams(char partitionMethod, char replicationModel);
 extern List * CitusTableList(void);
 extern ShardInterval * LoadShardInterval(uint64 shardId);
+extern bool ShardExists(uint64 shardId);
 extern Oid RelationIdForShard(uint64 shardId);
 extern bool ReferenceTableShardId(uint64 shardId);
+extern bool DistributedTableShardId(uint64 shardId);
 extern ShardPlacement * ShardPlacementOnGroupIncludingOrphanedPlacements(int32 groupId,
 																		 uint64 shardId);
 extern ShardPlacement * ActiveShardPlacementOnGroup(int32 groupId, uint64 shardId);
@@ -161,10 +171,10 @@ extern CitusTableCacheEntry * LookupCitusTableCacheEntry(Oid relationId);
 extern DistObjectCacheEntry * LookupDistObjectCacheEntry(Oid classid, Oid objid, int32
 														 objsubid);
 extern int32 GetLocalGroupId(void);
+extern int32 GetLocalNodeId(void);
 extern void CitusTableCacheFlushInvalidatedEntries(void);
 extern Oid LookupShardRelationFromCatalog(int64 shardId, bool missing_ok);
-extern List * ShardPlacementListIncludingOrphanedPlacements(uint64 shardId);
-extern bool ShardExists(int64 shardId);
+extern List * ShardPlacementList(uint64 shardId);
 extern void CitusInvalidateRelcacheByRelid(Oid relationId);
 extern void CitusInvalidateRelcacheByShardId(int64 shardId);
 extern void InvalidateForeignKeyGraph(void);
@@ -172,7 +182,6 @@ extern void FlushDistTableCache(void);
 extern void InvalidateMetadataSystemCache(void);
 extern List * CitusTableTypeIdList(CitusTableType citusTableType);
 extern Datum DistNodeMetadata(void);
-extern bool ClusterHasReferenceTable(void);
 extern bool HasUniformHashDistribution(ShardInterval **shardIntervalArray,
 									   int shardIntervalArrayLength);
 extern bool HasUninitializedShardInterval(ShardInterval **sortedShardIntervalArray,
@@ -185,11 +194,10 @@ extern bool HasOverlappingShardInterval(ShardInterval **shardIntervalArray,
 extern ShardPlacement * ShardPlacementForFunctionColocatedWithReferenceTable(
 	CitusTableCacheEntry *cacheEntry);
 extern ShardPlacement * ShardPlacementForFunctionColocatedWithDistTable(
-	DistObjectCacheEntry *procedure, FuncExpr *funcExpr, Var *partitionColumn,
+	DistObjectCacheEntry *procedure, List *argumentList, Var *partitionColumn,
 	CitusTableCacheEntry
 	*cacheEntry,
 	PlannedStmt *plan);
-
 extern bool CitusHasBeenLoaded(void);
 extern bool CheckCitusVersion(int elevel);
 extern bool CheckAvailableVersion(int elevel);
@@ -197,6 +205,7 @@ extern bool InstalledAndAvailableVersionsSame(void);
 extern bool MajorVersionsCompatible(char *leftVersion, char *rightVersion);
 extern void ErrorIfInconsistentShardIntervals(CitusTableCacheEntry *cacheEntry);
 extern void EnsureModificationsCanRun(void);
+extern void EnsureModificationsCanRunOnRelation(Oid relationId);
 extern char LookupDistributionMethod(Oid distributionMethodOid);
 extern bool RelationExists(Oid relationId);
 extern ShardInterval * TupleToShardInterval(HeapTuple heapTuple,
@@ -214,12 +223,15 @@ extern WorkerNode * LookupNodeForGroup(int32 groupId);
 extern Oid CitusCatalogNamespaceId(void);
 
 /* relation oids */
+extern Oid DistCleanupRelationId(void);
 extern Oid DistColocationRelationId(void);
 extern Oid DistColocationConfigurationIndexId(void);
 extern Oid DistPartitionRelationId(void);
 extern Oid DistShardRelationId(void);
 extern Oid DistPlacementRelationId(void);
 extern Oid DistNodeRelationId(void);
+extern Oid DistBackgroundJobRelationId(void);
+extern Oid DistBackgroundTaskRelationId(void);
 extern Oid DistRebalanceStrategyRelationId(void);
 extern Oid DistLocalGroupIdRelationId(void);
 extern Oid DistObjectRelationId(void);
@@ -229,14 +241,28 @@ extern Oid DistEnabledCustomAggregatesId(void);
 extern Oid DistNodeNodeIdIndexId(void);
 extern Oid DistPartitionLogicalRelidIndexId(void);
 extern Oid DistPartitionColocationidIndexId(void);
+extern Oid DistBackgroundJobPKeyIndexId(void);
+extern Oid DistBackgroundTaskPKeyIndexId(void);
+extern Oid DistBackgroundTaskJobIdTaskIdIndexId(void);
+extern Oid DistBackgroundTaskStatusTaskIdIndexId(void);
+extern Oid DistBackgroundTaskDependRelationId(void);
+extern Oid DistBackgroundTaskDependTaskIdIndexId(void);
+extern Oid DistBackgroundTaskDependDependsOnIndexId(void);
 extern Oid DistShardLogicalRelidIndexId(void);
 extern Oid DistShardShardidIndexId(void);
 extern Oid DistPlacementShardidIndexId(void);
 extern Oid DistPlacementPlacementidIndexId(void);
+extern Oid DistColocationIndexId(void);
 extern Oid DistTransactionRelationId(void);
 extern Oid DistTransactionGroupIndexId(void);
 extern Oid DistPlacementGroupidIndexId(void);
 extern Oid DistObjectPrimaryKeyIndexId(void);
+extern Oid DistCleanupPrimaryKeyIndexId(void);
+
+/* sequence oids */
+extern Oid DistBackgroundJobJobIdSequenceId(void);
+extern Oid DistBackgroundTaskTaskIdSequenceId(void);
+extern Oid DistClockLogicalSequenceId(void);
 
 /* type oids */
 extern Oid LookupTypeOid(char *schemaNameSting, char *typeNameString);
@@ -247,9 +273,12 @@ extern Oid CitusReadIntermediateResultFuncId(void);
 Oid CitusReadIntermediateResultArrayFuncId(void);
 extern Oid CitusExtraDataContainerFuncId(void);
 extern Oid CitusAnyValueFunctionId(void);
-extern Oid PgTableVisibleFuncId(void);
-extern Oid CitusTableVisibleFuncId(void);
+extern Oid CitusTextSendAsJsonbFunctionId(void);
+extern Oid TextOutFunctionId(void);
+extern Oid RelationIsAKnownShardFuncId(void);
 extern Oid JsonbExtractPathFuncId(void);
+extern Oid JsonbExtractPathTextFuncId(void);
+extern Oid CitusDependentObjectFuncId(void);
 
 /* enum oids */
 extern Oid PrimaryNodeRoleId(void);
@@ -257,6 +286,21 @@ extern Oid SecondaryNodeRoleId(void);
 extern Oid CitusCopyFormatTypeId(void);
 extern Oid TextCopyFormatId(void);
 extern Oid BinaryCopyFormatId(void);
+extern Oid CitusJobStatusScheduledId(void);
+extern Oid CitusJobStatusRunningId(void);
+extern Oid CitusJobStatusCancellingId(void);
+extern Oid CitusJobStatusFinishedId(void);
+extern Oid CitusJobStatusCancelledId(void);
+extern Oid CitusJobStatusFailedId(void);
+extern Oid CitusJobStatusFailingId(void);
+extern Oid CitusTaskStatusBlockedId(void);
+extern Oid CitusTaskStatusRunnableId(void);
+extern Oid CitusTaskStatusRunningId(void);
+extern Oid CitusTaskStatusDoneId(void);
+extern Oid CitusTaskStatusErrorId(void);
+extern Oid CitusTaskStatusUnscheduledId(void);
+extern Oid CitusTaskStatusCancelledId(void);
+extern Oid CitusTaskStatusCancellingId(void);
 
 /* user related functions */
 extern Oid CitusExtensionOwner(void);
@@ -264,4 +308,8 @@ extern char * CitusExtensionOwnerName(void);
 extern char * CurrentUserName(void);
 extern const char * CurrentDatabaseName(void);
 
+/* connection-related functions */
+extern char * GetAuthinfoViaCatalog(const char *roleName, int64 nodeId);
+extern char * GetPoolinfoViaCatalog(int64 nodeId);
+
 #endif /* METADATA_CACHE_H */
