@@ -61,8 +61,11 @@ typedef struct TransactionProperties
 } TransactionProperties;
 
 
+extern bool AllowNestedDistributedExecution;
+extern bool SkipConstraintValidation;
 extern int MultiShardConnectionType;
 extern bool WritableStandbyCoordinator;
+extern bool AllowModificationsFromWorkersToReplicatedTables;
 extern bool ForceMaxQueryParallelization;
 extern int MaxAdaptiveExecutorPoolSize;
 extern int ExecutorSlowStartInterval;
@@ -138,15 +141,18 @@ extern void ExecuteQueryIntoDestReceiver(Query *query, ParamListInfo params,
 extern void ExecutePlanIntoDestReceiver(PlannedStmt *queryPlan, ParamListInfo params,
 										DestReceiver *dest);
 extern void SetLocalMultiShardModifyModeToSequential(void);
+extern void EnsureSequentialMode(ObjectType objType);
 extern void SetLocalForceMaxQueryParallelization(void);
 extern void SortTupleStore(CitusScanState *scanState);
 extern bool DistributedPlanModifiesDatabase(DistributedPlan *plan);
 extern bool ReadOnlyTask(TaskType taskType);
+extern bool TaskListCannotBeExecutedInTransaction(List *taskList);
 extern void ExtractParametersFromParamList(ParamListInfo paramListInfo,
 										   Oid **parameterTypes,
 										   const char ***parameterValues, bool
 										   useOriginalCustomTypeOids);
 extern ParamListInfo ExecutorBoundParams(void);
+extern void EnsureTaskExecutionAllowed(bool isRemote);
 
 
 #endif /* MULTI_EXECUTOR_H */
