@@ -90,6 +90,9 @@ typedef struct MultiTable
 	Alias *referenceNames;
 	Query *subquery; /* this field is only valid for non-relation subquery types */
 	bool includePartitions;
+
+	/* FROM .. TABLESAMPLE clause */
+	TableSampleClause *tablesample;
 } MultiTable;
 
 
@@ -175,9 +178,7 @@ typedef struct MultiExtendedOp
 	List *sortClauseList;
 	Node *limitCount;
 	Node *limitOffset;
-#if PG_VERSION_NUM >= PG_VERSION_13
 	LimitOption limitOption;
-#endif
 	Node *havingQual;
 	List *distinctClause;
 	List *windowClause;
@@ -199,7 +200,6 @@ extern bool IsCitusTableRTE(Node *node);
 extern bool IsDistributedOrReferenceTableRTE(Node *node);
 extern bool IsDistributedTableRTE(Node *node);
 extern bool IsReferenceTableRTE(Node *node);
-extern bool QueryContainsDistributedTableRTE(Query *query);
 extern bool IsCitusExtraDataContainerRelation(RangeTblEntry *rte);
 extern bool ContainsReadIntermediateResultFunction(Node *node);
 extern bool ContainsReadIntermediateResultArrayFunction(Node *node);
