@@ -36,15 +36,6 @@
 /* Definitions local to the physical planner */
 #define NON_PRUNABLE_JOIN -1
 #define RESERVED_HASHED_COLUMN_ID MaxAttrNumber
-#define MERGE_COLUMN_FORMAT "merge_column_%u"
-#define MAP_OUTPUT_FETCH_COMMAND "SELECT worker_fetch_partition_file \
- (" UINT64_FORMAT ", %u, %u, %u, '%s', %u)"
-#define RANGE_PARTITION_COMMAND "SELECT worker_range_partition_table \
- (" UINT64_FORMAT ", %d, %s, '%s', '%s'::regtype, %s)"
-#define HASH_PARTITION_COMMAND "SELECT worker_hash_partition_table \
- (" UINT64_FORMAT ", %d, %s, '%s', '%s'::regtype, %s)"
-#define MERGE_FILES_INTO_TABLE_COMMAND "SELECT worker_merge_files_into_table \
- (" UINT64_FORMAT ", %d, '%s', '%s')"
 
 extern int RepartitionJoinBucketCountPerNode;
 
@@ -104,8 +95,7 @@ typedef enum
 {
 	JOB_INVALID_FIRST = 0,
 	JOIN_MAP_MERGE_JOB = 1,
-	SUBQUERY_MAP_MERGE_JOB = 2,
-	TOP_LEVEL_WORKER_JOB = 3
+	TOP_LEVEL_WORKER_JOB = 2
 } BoundaryNodeJobType;
 
 
@@ -161,6 +151,7 @@ typedef struct Job
 	 * query.
 	 */
 	bool parametersInJobQueryResolved;
+	uint32 colocationId; /* common colocation group ID of the relations */
 } Job;
 
 
@@ -168,7 +159,6 @@ typedef struct Job
 typedef struct MapMergeJob
 {
 	Job job;
-	Query *reduceQuery;
 	PartitionType partitionType;
 	Var *partitionColumn;
 	uint32 partitionCount;
@@ -261,6 +251,10 @@ typedef struct Task
 	uint32 upstreamTaskId;         /* only applies to data fetch tasks */
 	ShardInterval *shardInterval;  /* only applies to merge tasks */
 	bool assignmentConstrained;    /* only applies to merge tasks */
+
+	/* for merge tasks, this is set to the target list of the map task */
+	List *mapJobTargetList;
+
 	char replicationModel;         /* only applies to modify tasks */
 
 	/*
@@ -331,6 +325,11 @@ typedef struct Task
 	 * isLocalTableModification is true if the task is on modifying a local table.
 	 */
 	bool isLocalTableModification;
+
+	/*
+	 * Vacuum, create/drop/reindex concurrently cannot be executed in a transaction.
+	 */
+	bool cannotBeExecutedInTransction;
 } Task;
 
 
@@ -550,7 +549,6 @@ extern CollateExpr * RelabelTypeToCollateExpr(RelabelType *relabelType);
 extern Node * BuildBaseConstraint(Var *column);
 extern void UpdateConstraint(Node *baseConstraint, ShardInterval *shardInterval);
 extern bool BinaryOpExpression(Expr *clause, Node **leftOperand, Node **rightOperand);
-extern bool SimpleOpExpression(Expr *clause);
 
 /* helper functions */
 extern Var * MakeInt4Column(void);
