@@ -20,10 +20,13 @@
 
 extern void EnsureReferenceTablesExistOnAllNodes(void);
 extern void EnsureReferenceTablesExistOnAllNodesExtended(char transferMode);
+extern bool HasNodesWithMissingReferenceTables(List **referenceTableList);
 extern uint32 CreateReferenceTableColocationId(void);
-extern void DeleteAllReferenceTablePlacementsFromNodeGroup(int32 groupId);
+extern uint32 GetReferenceTableColocationId(void);
+extern void DeleteAllReplicatedTablePlacementsFromNodeGroup(int32 groupId,
+															bool localOnly);
 extern int CompareOids(const void *leftElement, const void *rightElement);
-extern int ReferenceTableReplicationFactor(void);
-extern void ReplicateAllReferenceTablesToNode(char *nodeName, int nodePort);
+extern void ReplicateAllReferenceTablesToNode(WorkerNode *workerNode);
+extern void ErrorIfNotAllNodesHaveReferenceTableReplicas(List *workerNodeList);
 
 #endif /* REFERENCE_TABLE_UTILS_H_ */
