@@ -20,6 +20,7 @@
 
 /* GUC, determining whether statements sent to remote nodes are logged */
 extern bool LogRemoteCommands;
+extern char *GrepRemoteCommands;
 
 /* GUC that determines the number of bytes after which remote COPY is flushed */
 extern int RemoteCopyFlushThreshold;
@@ -38,12 +39,15 @@ extern void ReportResultError(MultiConnection *connection, PGresult *result,
 							  int elevel);
 extern char * pchomp(const char *in);
 extern void LogRemoteCommand(MultiConnection *connection, const char *command);
+extern bool CommandMatchesLogGrepPattern(const char *command);
 
 /* wrappers around libpq functions, with command logging support */
 extern void ExecuteCriticalRemoteCommandList(MultiConnection *connection,
 											 List *commandList);
 extern void ExecuteCriticalRemoteCommand(MultiConnection *connection,
 										 const char *command);
+extern void ExecuteRemoteCommandInConnectionList(List *nodeConnectionList,
+												 const char *command);
 extern int ExecuteOptionalRemoteCommand(MultiConnection *connection,
 										const char *command,
 										PGresult **result);
@@ -64,4 +68,11 @@ extern void WaitForAllConnections(List *connectionList, bool raiseInterrupts);
 
 extern bool SendCancelationRequest(MultiConnection *connection);
 
+extern bool EvaluateSingleQueryResult(MultiConnection *connection, PGresult *queryResult,
+									  StringInfo queryResultString);
+extern void StoreErrorMessage(MultiConnection *connection, StringInfo queryResultString);
+
+extern bool IsSettingSafeToPropagate(const char *name);
+
+
 #endif /* REMOTE_COMMAND_H */
