@@ -16,6 +16,7 @@
 #include "distributed/worker_transaction.h"
 #include "nodes/pg_list.h"
 #include "storage/lock.h"
+#include "tcop/utility.h"
 
 
 /*
@@ -39,13 +40,21 @@ typedef enum AdvisoryLocktagClass
 	ADV_LOCKTAG_CLASS_CITUS_REBALANCE_COLOCATION = 7,
 	ADV_LOCKTAG_CLASS_CITUS_COLOCATED_SHARDS_METADATA = 8,
 	ADV_LOCKTAG_CLASS_CITUS_OPERATIONS = 9,
-	ADV_LOCKTAG_CLASS_CITUS_PLACEMENT_CLEANUP = 10
+	ADV_LOCKTAG_CLASS_CITUS_CLEANUP_OPERATION_ID = 10,
+	ADV_LOCKTAG_CLASS_CITUS_LOGICAL_REPLICATION = 12,
+	ADV_LOCKTAG_CLASS_CITUS_REBALANCE_PLACEMENT_COLOCATION = 13,
+	ADV_LOCKTAG_CLASS_CITUS_BACKGROUND_TASK = 14
 } AdvisoryLocktagClass;
 
 /* CitusOperations has constants for citus operations */
 typedef enum CitusOperations
 {
-	CITUS_TRANSACTION_RECOVERY = 0
+	CITUS_TRANSACTION_RECOVERY = 0,
+	CITUS_NONBLOCKING_SPLIT = 1,
+	CITUS_CREATE_DISTRIBUTED_TABLE_CONCURRENTLY = 2,
+	CITUS_CREATE_COLOCATION_DEFAULT = 3,
+	CITUS_SHARD_MOVE = 4,
+	CITUS_BACKGROUND_TASK_MONITOR = 5
 } CitusOperations;
 
 /* reuse advisory lock, but with different, unused field 4 (4)*/
@@ -72,14 +81,6 @@ typedef enum CitusOperations
 						 (uint32) (shardid), \
 						 ADV_LOCKTAG_CLASS_CITUS_SHARD)
 
-/* reuse advisory lock, but with different, unused field 4 (6) */
-#define SET_LOCKTAG_JOB_RESOURCE(tag, db, jobid) \
-	SET_LOCKTAG_ADVISORY(tag, \
-						 db, \
-						 (uint32) ((jobid) >> 32), \
-						 (uint32) (jobid), \
-						 ADV_LOCKTAG_CLASS_CITUS_JOB)
-
 /* reuse advisory lock, but with different, unused field 4 (7)
  * Also it has the database hardcoded to MyDatabaseId, to ensure the locks
  * are local to each database */
@@ -90,6 +91,17 @@ typedef enum CitusOperations
 						 (uint32) (colocationOrTableId), \
 						 ADV_LOCKTAG_CLASS_CITUS_REBALANCE_COLOCATION)
 
+/* reuse advisory lock, but with different, unused field 4 (13)
+ * Also it has the database hardcoded to MyDatabaseId, to ensure the locks
+ * are local to each database */
+#define SET_LOCKTAG_REBALANCE_PLACEMENT_COLOCATION(tag, colocationOrTableId) \
+	SET_LOCKTAG_ADVISORY(tag, \
+						 MyDatabaseId, \
+						 (uint32) ((colocationOrTableId) >> 32), \
+						 (uint32) (colocationOrTableId), \
+						 ADV_LOCKTAG_CLASS_CITUS_REBALANCE_PLACEMENT_COLOCATION)
+
+
 /* advisory lock for citus operations, also it has the database hardcoded to MyDatabaseId,
  * to ensure the locks are local to each database */
 #define SET_LOCKTAG_CITUS_OPERATION(tag, operationId) \
@@ -102,12 +114,53 @@ typedef enum CitusOperations
 /* reuse advisory lock, but with different, unused field 4 (10)
  * Also it has the database hardcoded to MyDatabaseId, to ensure the locks
  * are local to each database */
-#define SET_LOCKTAG_PLACEMENT_CLEANUP(tag) \
+#define SET_LOCKTAG_CLEANUP_OPERATION_ID(tag, operationId) \
+	SET_LOCKTAG_ADVISORY(tag, \
+						 MyDatabaseId, \
+						 (uint32) ((operationId) >> 32), \
+						 (uint32) operationId, \
+						 ADV_LOCKTAG_CLASS_CITUS_CLEANUP_OPERATION_ID)
+
+/* reuse advisory lock, but with different, unused field 4 (12)
+ * Also it has the database hardcoded to MyDatabaseId, to ensure the locks
+ * are local to each database */
+#define SET_LOCKTAG_LOGICAL_REPLICATION(tag) \
 	SET_LOCKTAG_ADVISORY(tag, \
 						 MyDatabaseId, \
 						 (uint32) 0, \
 						 (uint32) 0, \
-						 ADV_LOCKTAG_CLASS_CITUS_PLACEMENT_CLEANUP)
+						 ADV_LOCKTAG_CLASS_CITUS_LOGICAL_REPLICATION)
+
+/* reuse advisory lock, but with different, unused field 4 (14)
+ * Also it has the database hardcoded to MyDatabaseId, to ensure the locks
+ * are local to each database */
+#define SET_LOCKTAG_BACKGROUND_TASK(tag, taskId) \
+	SET_LOCKTAG_ADVISORY(tag, \
+						 MyDatabaseId, \
+						 (uint32) ((taskId) >> 32), \
+						 (uint32) (taskId), \
+						 ADV_LOCKTAG_CLASS_CITUS_BACKGROUND_TASK)
+
+/*
+ * DistLockConfigs are used to configure the locking behaviour of AcquireDistributedLockOnRelations
+ */
+enum DistLockConfigs
+{
+	/*
+	 * lock citus tables
+	 */
+	DIST_LOCK_DEFAULT = 0,
+
+	/*
+	 * lock tables that refer to locked citus tables with a foreign key
+	 */
+	DIST_LOCK_REFERENCING_TABLES = 1,
+
+	/*
+	 * throw an error if the lock is not immediately available
+	 */
+	DIST_LOCK_NOWAIT = 2
+};
 
 
 /* Lock shard/relation metadata for safe modifications */
@@ -125,16 +178,13 @@ extern void LockReferencedReferenceShardDistributionMetadata(uint64 shardId,
 /* Lock shard data, for DML commands or remote fetches */
 extern void LockShardResource(uint64 shardId, LOCKMODE lockmode);
 
-/* Lock a job schema or partition task directory */
-extern void LockJobResource(uint64 jobId, LOCKMODE lockmode);
-extern void UnlockJobResource(uint64 jobId, LOCKMODE lockmode);
-
 /* Lock a co-location group */
 extern void LockColocationId(int colocationId, LOCKMODE lockMode);
 extern void UnlockColocationId(int colocationId, LOCKMODE lockMode);
 
 /* Lock multiple shards for safe modification */
 extern void LockShardListMetadata(List *shardIntervalList, LOCKMODE lockMode);
+extern void LockShardListMetadataOnWorkers(LOCKMODE lockmode, List *shardIntervalList);
 extern void LockShardsInPlacementListMetadata(List *shardPlacementList,
 											  LOCKMODE lockMode);
 
@@ -144,11 +194,20 @@ extern void SerializeNonCommutativeWrites(List *shardIntervalList, LOCKMODE lock
 extern void LockRelationShardResources(List *relationShardList, LOCKMODE lockMode);
 extern List * GetSortedReferenceShardIntervals(List *relationList);
 
+void AcquireCreateDistributedTableConcurrentlyLock(Oid relationId);
+
 /* Lock parent table's colocated shard resource */
-extern void LockParentShardResourceIfPartition(uint64 shardId, LOCKMODE lockMode);
+extern void LockParentShardResourceIfPartition(List *shardIntervalList,
+											   LOCKMODE lockMode);
 
 /* Lock mode translation between text and enum */
 extern LOCKMODE LockModeTextToLockMode(const char *lockModeName);
 extern const char * LockModeToLockModeText(LOCKMODE lockMode);
+extern void AcquireDistributedLockOnRelations(List *relationList, LOCKMODE lockMode,
+											  uint32 configs);
+extern void PreprocessLockStatement(LockStmt *stmt, ProcessUtilityContext context);
+
+extern bool EnableAcquiringUnsafeLockFromWorkers;
+extern bool SkipAdvisoryLockPermissionChecks;
 
 #endif /* RESOURCE_LOCK_H */
