@@ -9,9 +9,16 @@
 #ifndef TRANSACTION_MANAGMENT_H
 #define TRANSACTION_MANAGMENT_H
 
+#include "access/xact.h"
 #include "lib/ilist.h"
 #include "lib/stringinfo.h"
 #include "nodes/pg_list.h"
+#include "lib/stringinfo.h"
+#include "nodes/primnodes.h"
+
+/* forward declare, to avoid recursive includes */
+struct DistObjectCacheEntry;
+
 
 /* describes what kind of modifications have occurred in the current transaction */
 typedef enum
@@ -46,14 +53,6 @@ typedef enum CoordinatedTransactionState
 } CoordinatedTransactionState;
 
 
-/* Enumeration that defines the different commit protocols available */
-typedef enum
-{
-	COMMIT_PROTOCOL_BARE = 0,
-	COMMIT_PROTOCOL_1PC = 1,
-	COMMIT_PROTOCOL_2PC = 2
-} CommitProtocolType;
-
 /* Enumeration to keep track of context within nested sub-transactions */
 typedef struct SubXactContext
 {
@@ -61,6 +60,50 @@ typedef struct SubXactContext
 	StringInfo setLocalCmds;
 } SubXactContext;
 
+/*
+ * Function delegated with force_delegation call enforces the distribution argument
+ * along with the colocationId. The latter one is equally important to not allow
+ * the same partition key value into another distributed table which is not co-located
+ * and therefore might be on a different node.
+ */
+typedef struct AllowedDistributionColumn
+{
+	Const *distributionColumnValue;
+	uint32 colocationId;
+	bool isActive;
+
+	/* In nested executor, track the level at which value is set */
+	int executorLevel;
+} AllowedDistributionColumn;
+
+/*
+ * BeginXactDeferrableState reflects the value of the DEFERRABLE property
+ * in the BEGIN of a transaction block.
+ */
+typedef enum BeginXactDeferrableState
+{
+	BeginXactDeferrable_NotSet,
+	BeginXactDeferrable_Disabled,
+	BeginXactDeferrable_Enabled,
+} BeginXactDeferrableState;
+
+/*
+ * BeginXactReadOnlyState reflects the value of the READ ONLY property
+ * in the BEGIN of a transaction block.
+ */
+typedef enum BeginXactReadOnlyState
+{
+	BeginXactReadOnly_NotSet,
+	BeginXactReadOnly_Disabled,
+	BeginXactReadOnly_Enabled,
+} BeginXactReadOnlyState;
+
+/*
+ * The current distribution column value passed as an argument to a forced
+ * function call delegation.
+ */
+extern AllowedDistributionColumn AllowedDistributionColumnValue;
+
 /*
  * GUC that determines whether a SELECT in a transaction block should also run in
  * a transaction block on the worker.
@@ -73,13 +116,6 @@ extern bool SelectOpensTransactionBlock;
  */
 extern bool FunctionOpensTransactionBlock;
 
-/* config variable managed via guc.c */
-extern int MultiShardCommitProtocol;
-extern int SingleShardCommitProtocol;
-
-/* state needed to restore multi-shard commit protocol during VACUUM/ANALYZE */
-extern int SavedMultiShardCommitProtocol;
-
 /* state needed to prevent new connections during modifying transactions */
 extern XactModificationType XactModificationLevel;
 
@@ -106,6 +142,10 @@ extern StringInfo activeSetStmts;
 /* did current transaction modify pg_dist_node? */
 extern bool TransactionModifiedNodeMetadata;
 
+/* after an explicit BEGIN, keep track of top-level transaction characteristics */
+extern BeginXactReadOnlyState BeginXactReadOnly;
+extern BeginXactDeferrableState BeginXactDeferrable;
+
 /*
  * Coordinated transaction management.
  */
@@ -115,6 +155,8 @@ extern void Use2PCForCoordinatedTransaction(void);
 extern bool GetCoordinatedTransactionShouldUse2PC(void);
 extern bool IsMultiStatementTransaction(void);
 extern void EnsureDistributedTransactionId(void);
+extern bool MaybeExecutingUDF(void);
+
 
 /* initialization function(s) */
 extern void InitializeTransactionManagement(void);
@@ -122,7 +164,7 @@ extern void InitializeTransactionManagement(void);
 /* other functions */
 extern List * ActiveSubXactContexts(void);
 extern StringInfo BeginAndSetDistributedTransactionIdCommand(void);
-extern void TriggerMetadataSyncOnCommit(void);
+extern void TriggerNodeMetadataSyncOnCommit(void);
 
 
 #endif /*  TRANSACTION_MANAGMENT_H */
